import numpy as np
import torch
from dataset import IAN_Dataset
from torch.utils.data import DataLoader
from models import IAN
from utils import collate_helper, evaluate_error
import argparse

parser = argparse.ArgumentParser()
parser.add_argument('-d', '--dataset', type=str, required=True)
parser.add_argument('-bm', '--base_model', type=str, required=True)
parser.add_argument('-mp', '--model_path', type=str, required=True)
args = parser.parse_args()

dataset_name = args.dataset
base_model = args.base_model
test_dataset = IAN_Dataset(dataset_name, "test", base_model)
test_loader = DataLoader(test_dataset, batch_size=32, num_workers=8, collate_fn=collate_helper)

if dataset_name == "NBA":
    net = IAN(obs_len=5, pred_len=10).cuda()
else:
    net = IAN(obs_len=8, pred_len=12).cuda()

model_path = args.model_path
ckpt = torch.load(model_path)
net.load_state_dict(ckpt)
print("Network Loaded", model_path)

net.set_base_model(base_model)
print("Begin Testing", base_model, dataset_name)

original_ade = []
original_fde = []
adjusted_ade = []
adjusted_fde = []

net.eval()
for batch in test_loader:
    batch_obs, batch_gt, batch_base_model_pred, batch_sizes, \
        batch_feedback_trj_obs, batch_feedback_trj_gt, batch_feedback_predictions, _ = batch

    batch_obs = batch_obs.cuda()
    batch_gt = batch_gt.cuda()
    batch_base_model_pred = batch_base_model_pred.cuda()
    batch_feedback_trj_obs, batch_feedback_trj_gt = batch_feedback_trj_obs.cuda(), batch_feedback_trj_gt.cuda()
    batch_feedback_predictions = batch_feedback_predictions.cuda()
    batch_feedback_trj = torch.cat([batch_feedback_trj_obs, batch_feedback_trj_gt], dim=-2)[..., 1:]
    adjusted_predictions = net.adjust(batch_obs, batch_base_model_pred,
                                      batch_feedback_trj, batch_feedback_predictions, batch_sizes)

    ade, fde = evaluate_error(batch_base_model_pred, batch_gt)
    original_ade += ade
    original_fde += fde

    ade, fde = evaluate_error(adjusted_predictions, batch_gt)
    adjusted_ade += ade
    adjusted_fde += fde

print("Test Original Eval ADE & FDE: %.4f %.4f" % (np.sum(original_ade) / len(test_dataset), np.sum(original_fde) / len(test_dataset)))
print("Test Adjusted Eval ADE & FDE: %.4f %.4f" % (np.sum(adjusted_ade) / len(test_dataset), np.sum(adjusted_fde) / len(test_dataset)))

