import torch
import numpy as np
import os
import glob


def collate_helper(data):
    obs_trj = []
    pred_gt = []
    base_model_pred = []
    sizes = []
    feedback_base_prediction = []
    feedback_trj_obs = []
    feedback_trj_gt = []
    base_model_pred_error = []

    for i in range(len(data)):
        obs_trj.append(torch.from_numpy(data[i][0]).float())
        gt = torch.from_numpy(data[i][1]).float()
        base_pred = torch.from_numpy(data[i][2]).float()
        feedback_base_prediction.append(torch.from_numpy(data[i][3]).float())
        feedback_trj_obs.append(torch.from_numpy(data[i][4]).float())
        feedback_trj_gt.append(torch.from_numpy(data[i][5]).float())
        base_model_pred_error.append(torch.from_numpy(data[i][6]).float())
        pred_gt.append(gt)
        base_model_pred.append(base_pred)
        sizes.append(len(data[i][4]))

    obs_trj = torch.stack(obs_trj)
    pred_gt = torch.stack(pred_gt)
    base_model_pred = torch.stack(base_model_pred)
    feedback_trj_obs = torch.cat(feedback_trj_obs, dim=0)
    feedback_trj_gt = torch.cat(feedback_trj_gt, dim=0)
    feedback_base_prediction = torch.cat(feedback_base_prediction, dim=0)
    base_model_pred_error = torch.stack(base_model_pred_error, dim=0)

    return obs_trj, pred_gt, base_model_pred, sizes, \
           feedback_trj_obs, feedback_trj_gt, feedback_base_prediction, base_model_pred_error


def get_base_model_output(base_model_name, dataset_name, split):
    record = dict()
    base_dir = "%s_%s_%s_output" % (dataset_name, base_model_name, split)
    gt = np.load(os.path.join(base_dir, "gt.npy"))
    predictions = np.load(os.path.join(base_dir, "pred.npy"))
    rec = np.load(os.path.join(base_dir, "record.npy"))
    for i, info in enumerate(rec):
        record[(info[0], info[2], info[3])] = info[1]
    return predictions, gt, record


def get_raw_data(dataset_name, split):
    step_size = 10
    data_dir = os.path.join("dataset", dataset_name, split)
    raw_data = []
    for file_name in glob.glob(os.path.join(data_dir, "*.txt")):
        raw_data.append(np.genfromtxt(file_name))
    return raw_data, step_size


def evaluate_error(predictions, gt):
    ade = []
    fde = []
    for i in range(len(predictions)):
        diff = torch.norm(predictions[i] - gt[i:i + 1], dim=-1)
        error = diff.mean(-1)
        idx = torch.argmin(error)
        ade.append(error[idx].item())
        error = diff[:, -1]
        idx = torch.argmin(error)
        fde.append(diff[idx, -1].item())
    return ade, fde
